using System;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Threading;
using System.Xml;

using Ion.MarketView.Mmi.Plugin.Components;
using Ion.MarketView.Mmi.Plugin.Components.InstrumentChooser;
using Ion.MarketView.Mmi.Plugin.DataManagement;
using Ion.MarketView.Mmi.Plugin.MarketProperties;
using Ion.MarketView.Mmi.Plugin.OrderManagement;
using Ion.MarketView.Mmi.Plugin.Preferences;
using Ion.MarketView.Mmi.Plugin.QuoteManagement;
using Ion.MarketView.Mmi.Plugin.RfqManagement;
using Ion.MarketView.Mmi.Plugin.Toolbars;
using Ion.MarketView.Mmi.Plugin;
using Ion.MarketView.Ifc;

using Syncfusion.Drawing;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Grid.Grouping;

using Citi.Cache;
using Citi.IonWrapper;
using Citi.DataGridHelper;
using Citi.Tools;

namespace Citi.SpreadEngine.Listeners
{
	/// <summary>
	/// Summary description for SpreadEngineListener.
	/// </summary>
	public class SpreadEngineListener : MultipleTransactionChainListener
	{
		private SyncfusionDataBoundGrid _dataGrid;
		private Cache.Cache _cache;
		private Hashtable _spreadDataGridInfo;

		public SpreadEngineListener(CustomRecordDefinition recDef_, string source_, string serviceDescription_, Object context_, string chainName_, IDataGrid dataGrid_)  : 
			base(recDef_, source_, serviceDescription_, context_, chainName_)
		{
			_dataGrid = dataGrid_ as SyncfusionDataBoundGrid;
			_cache = Cache.Cache.Instance;
			Hashtable dataGridInfo = (Hashtable)_cache.RetrieveObject("DATA_GRID_INFO");
			if(dataGridInfo == null)
			{
				dataGridInfo = new Hashtable();
				_cache.AddObject("DATA_GRID_INFO", dataGridInfo);
			}
			// Now get info for this specific grid
			_spreadDataGridInfo = (Hashtable)dataGridInfo[_dataGrid.Name];
			if(_spreadDataGridInfo == null)
			{
				_spreadDataGridInfo = new Hashtable();
				dataGridInfo.Add(_dataGrid.Name, _spreadDataGridInfo);
			}
			_dataGrid.PrepareViewStyleInfo += new GridPrepareViewStyleInfoEventHandler(_dataGrid_PrepareViewStyleInfo);
			_dataGrid.CurrentCellCloseDropDown += new PopupClosedEventHandler(_dataGrid_CurrentCellCloseDropDown);
			
			///GridSettings();
			GridSettings2();
			PopulateDataGrid();
			ConfigHeaders();
		}

		/// <summary>
		/// Below copies settings given in MultiHeaderExample
		/// </summary>
		private void GridSettings2()
		{
			_dataGrid.UseListChangedEvent = true;
			_dataGrid.AllowDragSelectedCols = true;
			_dataGrid.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			_dataGrid.HorizontalThumbTrack = true;
			_dataGrid.ShowCurrentCellBorderBehavior = Syncfusion.Windows.Forms.Grid.GridShowCurrentCellBorder.GrayWhenLostFocus;
			_dataGrid.SmartSizeBox = false;
			_dataGrid.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;
			_dataGrid.ThemesEnabled = true;
			_dataGrid.VerticalScrollTips = true;
			_dataGrid.VerticalThumbTrack = true;
			_dataGrid.CurrentCellChanged +=new EventHandler(_dataGrid_CurrentCellChanged);
		}

		private void GridSettings()
		{
			
			_dataGrid.Model.Options.ShowCurrentCellBorderBehavior = GridShowCurrentCellBorder.WhenGridActive;
			_dataGrid.Model.Options.ActivateCurrentCellBehavior = GridCellActivateAction.SelectAll;
			_dataGrid.Model.Options.RefreshCurrentCellBehavior = GridRefreshCurrentCellBehavior.RefreshRow;
			_dataGrid.SortBehavior = Syncfusion.Windows.Forms.Grid.GridSortBehavior.DoubleClick;

			_dataGrid.ForceUpdateAfterListChangedEvent = true;
			_dataGrid.OptimizeDrawBackground = true;
			_dataGrid.OptimizeInsertRemoveCells = false;
			_dataGrid.Properties.BackgroundColor = System.Drawing.Color.WhiteSmoke;
			_dataGrid.Properties.BackgroundColorString = "WhiteSmoke";
			_dataGrid.Properties.ColHeaders = true;
			_dataGrid.Properties.FixedLinesColorString = "ActiveCaption";
			_dataGrid.Properties.GridLineColor = System.Drawing.SystemColors.ControlDarkDark;
			_dataGrid.Properties.GridLineColorString = "ControlDarkDark";
			_dataGrid.Properties.ResizingCellsLinesColorString = "Red";
			// Below settings gets rid of border on LHS of grid
			////_dataGrid.Properties.RowHeaders = false;
			_dataGrid.ListBoxSelectionMode = SelectionMode.One;
			_dataGrid.HorizontalThumbTrack = true;

			// Resize rows to fit height
			_dataGrid.Model.ColWidths.ResizeToFit(GridRangeInfo.Cells(0, 0, 2, _dataGrid.Model.ColCount), GridResizeToFitOptions.None);
			_dataGrid.Model.RowHeights.ResizeToFit(GridRangeInfo.Cells(0, 0, 2, _dataGrid.Model.ColCount), GridResizeToFitOptions.None);
			_dataGrid.AllowResizeToFit = false;

			// Resize rows option
			_dataGrid.Model.Options.ResizeRowsBehavior = GridResizeCellsBehavior.ResizeSingle|GridResizeCellsBehavior.OutlineBounds|GridResizeCellsBehavior.OutlineHeaders;
			_dataGrid.Model.Options.ResizeColsBehavior = GridResizeCellsBehavior.ResizeSingle|GridResizeCellsBehavior.OutlineBounds|GridResizeCellsBehavior.OutlineHeaders;

			// Set fond bold for standard column headers
			GridStyleInfo gsiHeader = _dataGrid.Model.BaseStylesMap["Header"].StyleInfo ;
			gsiHeader.Font.Bold = true;
			gsiHeader.Interior = new BrushInfo(GradientStyle.Vertical, Color.FromArgb( 192, 201, 219 ), Color.FromArgb( 237, 240, 246 ));
			gsiHeader.TextColor = Color.FromArgb(51, 51, 102);
		}

		private void ConfigHeaders()
		{
			_dataGrid.BeginUpdate();
			_dataGrid.Model.Rows.HeaderCount = 2;
			_dataGrid.Model.Rows.FrozenCount = 2;

			_dataGrid.Model.CoveredRanges.Add(GridRangeInfo.Cells(0, 3, 0, 4));
			_dataGrid[0, 3].Text = "Core";

			// Before saving cell values we need to initialize internal data store to accept cell information
			// Otherwise, we would get an "Out of Range" exception when saving cells.
			_dataGrid.Model.Data.RowCount = _dataGrid.Model.Rows.HeaderCount+1;
			_dataGrid.Model.Data.ColCount = _dataGrid.Model.ColCount;

			// Now save the cells that should be displayed in the header cells.
			int maxCol = 4;
			GridStyleInfo[] ar = new GridStyleInfo[maxCol];
			for (int n = 0; n < maxCol; n++)
			{
				GridStyleInfo style = new GridStyleInfo();
				style.BaseStyle = "Standard"; // change base style - we don't want the standard "Header" basestyle.
				style.Borders.Bottom = new GridBorder(GridBorderStyle.Dashed, Color.Gray, GridBorderWeight.Medium);
				
				ar[n] = style;

			}

			////_dataGrid.Model.ChangeCells(GridRangeInfo.Cells(2, 1, 2, maxCol), ar);
			
			_dataGrid.EndUpdate();
		}

		private void PopulateDataGrid()
		{
			Cursor.Current = Cursors.WaitCursor;

			DataTable dt = (DataTable)_dataGrid.DataSource;
			_dataGrid.BeginUpdate();
			DataRow dr;
			
			for(int i = 0; i < 50; i++)
			{

				dr = dt.NewRow();
				dr["Instrument"] = "Instrument " + i;
				dr["Liquidity"] = 1.00 + i;
				dt.Rows.Add(dr);

				// Add ComboBox items to cache so can be fetched in PrepareViewStyleInfo
				StringCollection coreStrategyValues = new StringCollection();
				coreStrategyValues.Add("Strategy A");
				coreStrategyValues.Add("Strategy B");
				coreStrategyValues.Add("Strategy C");
				coreStrategyValues.Add("Strategy D");
				coreStrategyValues.Add("Strategy E");
				coreStrategyValues.Add("Strategy F");

				StringCollection coreSizeValues = new StringCollection();
				coreSizeValues.Add("Size A");
				coreSizeValues.Add("Size B");
				coreSizeValues.Add("Size C");

				Hashtable instrumentInfo = new Hashtable();
				instrumentInfo.Add("CoreStrategyValues", coreStrategyValues);
				instrumentInfo.Add("CoreSizeValues", coreSizeValues);

				// If previous entry for instrument overwrite
				_spreadDataGridInfo["Instrument " + i] = instrumentInfo;
			}
			_dataGrid.EndUpdate(true);

			Cursor.Current = Cursors.Default;
		}

		/// <summary>
		/// This class is used to prepare ComboBox's, CheckBox's etc
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _dataGrid_PrepareViewStyleInfo(object sender, GridPrepareViewStyleInfoEventArgs e)
		{
			if (e.RowIndex == 0) 
			{
				return;
			}
			// Alternate colors below
			if(e.RowIndex >0 && e.RowIndex%2 == 0 && e.ColIndex > 0)
				e.Style.BackColor =  Color.PeachPuff;

			GridDataBoundGrid gdbg = sender as GridDataBoundGrid;
			int instrumentColIndex = gdbg.Binder.NameToField("Instrument");
			int coreStrategyColIndex = gdbg.Binder.NameToField("CoreStrategy");
			int coreSizeColIndex = gdbg.Binder.NameToField("CoreSize");

			// Get back info for instrument's row
			string instrument = (string)gdbg[e.RowIndex, instrumentColIndex+1].CellValue;
			Hashtable instrumentInfo = (Hashtable)_spreadDataGridInfo[instrument];

			if( gdbg[e.RowIndex, e.ColIndex] != null  &&  e.ColIndex == coreStrategyColIndex+1 )
			{
				// For now just put standard items in drop down
				StringCollection coreStrategyValues = (StringCollection)instrumentInfo["CoreStrategyValues"];
				string coreStrategySelectedItem = instrumentInfo["CoreStrategySelectedItem"] as string;
				e.Style.ChoiceList = coreStrategyValues;
				e.Style.CellType = "ComboBox";
				if(coreStrategySelectedItem == null)
				{
					e.Style.CellValue = coreStrategyValues[e.RowIndex % 6];
				}
				else
				{
					e.Style.CellValue = coreStrategySelectedItem;
				}
				return;
			}
			if( gdbg[e.RowIndex, e.ColIndex] != null  &&  e.ColIndex == coreSizeColIndex+1 )
			{
				// For now just put standard items in drop down
				StringCollection coreSizeValues = (StringCollection)instrumentInfo["CoreSizeValues"];
				string coreStrategySizeItem = instrumentInfo["CoreSizeSelectedItem"] as string;
				e.Style.ChoiceList = coreSizeValues;
				e.Style.CellType = "ComboBox";
				if(coreStrategySizeItem == null)
				{
					e.Style.CellValue = coreSizeValues[e.RowIndex % 3];
				}
				else
				{
					e.Style.CellValue = coreStrategySizeItem;
				}
				return;
			}
		}

		private void _dataGrid_CurrentCellCloseDropDown(object sender, PopupClosedEventArgs e)
		{
			GridDataBoundGrid dataGrid = sender as GridDataBoundGrid;					
			GridCurrentCell cc = dataGrid.CurrentCell;			
			DataTable dt = (DataTable)_dataGrid.DataSource;
			DataRow dataRow = dt.Rows[cc.RowIndex];		
			string mappingName = dt.Columns[cc.ColIndex - 1].ColumnName;
			// Get intrument info from cache
			int instrumentColIndex = dataGrid.Binder.NameToField("Instrument");
			string instrument = (string)dataGrid[cc.RowIndex, instrumentColIndex+1].CellValue;
			Hashtable instrumentInfo = (Hashtable)_spreadDataGridInfo[instrument];


			GridComboBoxCellRenderer cr = cc.Renderer as GridComboBoxCellRenderer; 
			if(cr != null)
			{
				string item = (string)cr.ListBoxPart.SelectedItem; 
				if(item != null)
				{
					// Need to change the display item in the Hashtable so it updates in PrepareView
					if(mappingName == "CoreStrategy")
					{
						instrumentInfo["CoreStrategySelectedItem"] = item;
					}
					else if(mappingName == "CoreSize")
					{
						instrumentInfo["CoreSizeSelectedItem"] = item;
					}
				}
			}
			cc.Refresh();
		}

		private void _dataGrid_CurrentCellChanged(object sender, EventArgs e)
		{
//			// Set current position and/or repaint row when user clicked the checkbox in the row header
//			GridDataBoundGrid grid = (GridDataBoundGrid) sender;
//			int colIndex = grid.CurrentCell.ColIndex;
//			int rowIndex = grid.CurrentCell.RowIndex;
//			if (colIndex == 1)
//			{
//				int record = gridBinder.RowIndexToPosition(rowIndex);
//				if (gridBinder.CurrentPosition != record)
//					gridBinder.CurrentPosition = record;
//				else
//					gridModel.InvalidateRange(GridRangeInfo.Row(rowIndex), GridRangeOptions.None);
//			}
		}
	}
}
