using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Collections.Specialized;
using System.Reflection;
using System.IO;
using System.Xml;
using System.Threading;
using System.Security.Cryptography;

using Microsoft.Win32;

using Ion.MarketView.Mmi.Plugin.Components;
using Ion.MarketView.Mmi.Plugin.Components.InstrumentChooser;
using Ion.MarketView.Mmi.Plugin.DataManagement;
using Ion.MarketView.Mmi.Plugin.MarketProperties;
using Ion.MarketView.Mmi.Plugin.OrderManagement;
using Ion.MarketView.Mmi.Plugin.Preferences;
using Ion.MarketView.Mmi.Plugin.QuoteManagement;
using Ion.MarketView.Mmi.Plugin.RfqManagement;
using Ion.MarketView.Mmi.Plugin.Toolbars;
using Ion.MarketView.Mmi.Plugin;
using Ion.MarketView.Ifc;

using Citi.IonWrapper;
using Citi.DataGridHelper;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Tools;

namespace Citi.SpreadEngine
{
	/// <summary>
	/// Summary description for SpreadEnginePlugin.
	/// </summary>
	[System.Runtime.InteropServices.Guid("5676DB80-72CF-40dc-BADF-35AA7DE42081")]
	[Plugin("Spread Engine Plugin")]
	public class SpreadEnginePlugin : CitiPluginBase
	{
		internal ICustomComponent spreadEngineDataGridComponent;
		internal DataGridStore dgStore;

		private string configDir;
		private SyncfusionDataBoundGrid _sfbg;

		public const string SPREAD_DATA_GRID_COMPONENT_NAME = "Spread Engine Data Grid";

		public SpreadEnginePlugin()
		{			
			RegistryKey hklm = Registry.LocalMachine;
			hklm = hklm.OpenSubKey("SOFTWARE\\ION Trading\\MMI.NET\\");
			Object obp = hklm.GetValue("InstallDir");
			
			configDir = Citi.IonWrapper.PluginContext.RetrieveInstallationDir();
			configDir = configDir + "\\plugins";
		}

		protected override void ProcessComponentCreated(object sender, Ion.MarketView.Mmi.Plugin.Components.ComponentCreatedEventArgs args)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(@"C:\Documents and Settings\ch92071\My Documents\Visual Studio Projects\Citi.SpreadEngine\Config\Spread_Data_Grid.xml");
			switch (args.Component.Name)
			{
				case SPREAD_DATA_GRID_COMPONENT_NAME:
					spreadEngineDataGridComponent = (ICustomComponent) args.Component;
					_sfbg = (SyncfusionDataBoundGrid) spreadEngineDataGridComponent.InnerControl;
					string xpath = DataGridConstants.DATA_GRID_COLLECTION + "/" + DataGridConstants.DATA_GRID + "[@" + DataGridConstants.ATT_NAME + " = '" + "SpreadEngineGrid" + "']";
					_sfbg.Initialize((XmlNode)doc.DocumentElement.SelectSingleNode(xpath));
					spreadEngineDataGridComponent.Title = "Spread Engine";
					break;
			}

		}

		protected override void CustomInitialisation(IValuedParameterCollection parameters)
		{
			CustomRecordDefinitionMap = Citi.IonWrapper.CustomRecordManager.LoadCustomRecordDefinitions(@"C:\Documents and Settings\ch92071\My Documents\Visual Studio Projects\Citi.SpreadEngine\Config\Spread_Data_Grid.xml");
			dgStore = DataGridStore.Instance;
			dgStore.PopulateRecordDefinitions(CustomRecordDefinitionMap);				
			if (Ion.MarketView.Ifc.Application.Instance.IsConnected) 
			{		
				XmlDocument doc = new XmlDocument();
				doc.Load(@"C:\Documents and Settings\ch92071\My Documents\Visual Studio Projects\Citi.SpreadEngine\Config\Spread_Data_Grid.xml");
				dgStore.Init((XmlNode)doc.DocumentElement.SelectSingleNode(DataGridConstants.DATA_GRID_COLLECTION));
			

				_sfbg = dgStore.GetDataGrid("SpreadEngineGrid") as SyncfusionDataBoundGrid;
				spreadEngineDataGridComponent = Application.FindActiveWindow().CreateComponent(SPREAD_DATA_GRID_COMPONENT_NAME, _sfbg);
				spreadEngineDataGridComponent.Title = "Spread Engine";
				spreadEngineDataGridComponent.Serializable = true;
			}

		}
	}
}
